# Venom player
Пример:
```html
<script src="https://cdn.jsdelivr.net/npm/venom-player"></script>
<script>
VenomPlayer.make({
    publicPath: 'https://cdn.jsdelivr.net/npm/venom-player/dist/',
    source: {
        file: {
            360: 'https://raw.githubusercontent.com/mnaseersj/BigBuckBunny/master/BigBuckBunny_640x360.mp4'
        }
    }
})
</script>
```

## Опции
`publicPath` (_String_) задаёт базовый путь, откуда будут загружаться темы.
Например, если плеер подключен из https://cdn.jsdelivr.net/npm/venom-player,
то нужно указать `"https://cdn.jsdelivr.net/npm/venom-player/dist/"`

`source` (_Object_) комплексная опция, в основном для указания пути к 
источнику видео. Должна содержать хотя бы одну из секций:

* `dash` (_String_) путь к dash манифесту
* `hls` (_String_) путь к hls манифесту
* `hlsList` (_Object<Number,String>_) объект, в котором ключом выступает
качество видео, а значением - путь к hls манифесту для этого качества
* `file` (_Object<Number,String>_) так же, как и hlsList, но для
не стриминговых типов видео, а в виде одного файла.
Это может быть mp4, webm и т.д.

Эти опции должны представлять альтернативные варианты одного и того
же видео. Если указан `dash`, но он не поддерживается браузером
пользователя, то будет использован `hls`/`hlsList`; если же нет
поддержки hls (библиотеки hls.js или же нативной), воспроизводиться
будет `file`

Для устройств, поддерживаемых hls только нативно (в основном это
мобильные c iOS и некоторые модели смарт ТВ) лучше использовать
`hlsList` вместо `hls`, так как это позволит пользователям
самостоятельно выбрать наиболее удобное качество видео

Пример:
```javascript
opts = {
  // ...
  source: {
    // если используется Kaltura, то мастер манифест
    // hls: 'https://video.example/id/master.m3u8',
    // легко разделить по качествам с помощью специального роута `/tracks`
    hlsList: {
      480: 'https://video.example/id/tracks/v2-a/master.m3u8',
      720: 'https://video.example/id/tracks/v1-a/master.m3u8'
    }
  }
}
```

***
* `source.audio` позволяет переименовывать звуковые дорожки и изменять их порядок в меню
* `source.cc` субтитры
```javascript
opts = {
  // ...
  source: {
    // ...
    audio:  {
      names: ["Оригинал", "Дубляж"],
      order: [1, 0] // в меню будет "Дубляж", а затем "Оригинал" 
    },
    cc: {
      { name: "rus", url: "https://example.cc/rus.vtt" },
      { name: "eng", url: "https://example.cc/eng.vtt" }
    }
  }
}
```

`container` (_Element_) - ссылка на DOM элемент, в который следует встроить
плеер. Если не указан, будет использовано `document.body`.
Перед встраиванием весь контент контейнера будет очищен.
```javascript
container: document.getElementById('player-container')
```

`title` (_String_) - название видео. Не отображается в теме "classic"
```javascript
title: 'Game of Thrones'
```

`ui.titleOnlyOnFullscreen` (_Boolean_) если включена, то название видео будет
отображаться только в полноэкранном режиме
```javascript
ui: {
    titleOnlyOnFullscreen: true
}
```

`poster` (_String_) путь к постеру. Подробнее про poster [тут](https://developer.mozilla.org/en-US/docs/Web/HTML/Element/video)

`defaultPoster` (_String_) заглушка, которая будет использована как постер,
если изображение из опции `poster` по каким-либо причинам будет недоступно.

`autoLandscape` (_Boolean_) если установить `true`, то на мобильных при входе в
полноэкранный режим также будет использована альбомная ориентация экрана

`pip` (_Boolean | Number_) `true` - добавить кнопку "picture in picture",
по умолчанию `false`. При значении `0.5` переход в этот режим будет происходить
автоматически, когда видимость плеера станет ниже 50%

`live` (_Boolean_) для трансляций следует указать `live: true`

`liveBuffer` соответствует настройке *hls.js* `maxBufferLength`

`theme` (_String_) тема, в данный момент доступны "modern", "classic", "metro".
По умолчанию "venom"

<a name="css-vars"></a>
`cssVars` (_Object_) позволяет более тонко настроить вид плеера. Значения можно
обновить после инициализации с помощью [`VenomPlayer.setVariables({ ... })`](#set-variables)
TODO list

`aspectRatio` (_String_) соотношение сторон, по умолчанию `"16:9"`.
Значение `"fill"` (заполнить всё доступное пространство) или "ширина:высота"
(4:3, 10:9, 1:1...)

`blocked` (_Boolean_) если установлено в `true`, вместо плеера будет выведено
окно-заглушка с сообщением, что видео заблокировано. Текст сообщения можно
изменить с помощью `text.blocked`

`quality` (_Number_) качество по умолчанию
```javascript
quality: 720
```

`restrictQuality` (_Function_) позволяет ограничить качество. Вместо смены будет
выведено сообщение, что вернула функция. Если результат в логическом контексте
ложен - ограничений нет.
```javascript
restrictQuality: function(quality) {
  if (quality > 9000) {
    return "Your video card are not prepared!"
  }
}
```

`speed` (_Number[]_) список значений, из которых пользователь сможет выбрать
скорость воспроизведения
```javascript
speed: [1, 1.1, 1.25, 1.5]
```

`restrictSpeed` (_Function_) позволяет ограничить изменение скорости
воспроизведения, в зависимости от качества
```javascript
restrictSpeed: function(rate, quality) {
  if (rate > 1 && quality > 480) {
    return 'Ускорение доступно только для низкого качества видео'
  }
}
```

`volume` (_Number_) звук в пределах от `0` до `1`. По умолчанию `1`

`time` (_Number_) начать воспроизведения с указанного времени в секундах

`timeSearchParamName` (_String_) название get параметра, с которого будет взято
 значение `time`, по умолчанию `"t"`
 
`trackProgress` (_Number_) интервал в секундах, по которому будет срабатывать 
событие `viewProgress`, по умолчанию `60`

`doNotSaveProgress` (_Boolean_) if `true` then don't save progress to localStorage,
по умолчанию `false`

`rewind` (_Number[]_) время перемотки в секундах, по умолчанию `[5, 20]`. Первое
значение используется при перемотке стрелками клавиатуры и тапом на мобильном
(можно несколько раз подряд), второе - с зажатой кнопкой _shift_ и на телевизоре

`replay` повторять воспроизведение

`reportUrl` (_String_) url, на который будет отправляться обратная связь TODO

`dash` (_Object_) настройки dashjs, [подробнее](http://cdn.dashjs.org/latest/jsdoc/module-MediaPlayer.html#updateSettings__anchor)

```javascript
//TODO
ui: {
    lock: true, // показывать кнопку блокировки на мобильных, по умолчанию false
    // share: false, // спрятать кнопку поделиться, по умолчанию true
    // share: ['facebook', 'vkontakte', 'odnoklassniki'], // белый список
    // timeline: false, // по умолчанию true
    // fullscreen: 'external'|false, по умолчанию true
}
```

`text`, `translations` изменить [надписи](#text)
```javascript
text: {
    settings: 'Настройки'
},
translations: { // позволяет изменить или дополнить переводы
    en: {
        settings: '[ Settings ]'
    }
}
```

`preview` TODO

`oneSound` (_String_) позволяет спрятать все звуковые дорожки, кроме указанной
```javascript
oneSound: 'original' // регистронезависимо; можно указать лишь часть названия
```

`soundBlock` (_String_) спрятать перечисленные звуковые дорожки
```javascript
soundBlock: 'spanish,одноголосый' // можно указать лишь часть названия
```


## События
Поддерживаются стандартные [медиа события](https://developer.mozilla.org/en-US/docs/Web/Guide/Events/Media_events)
и события VPAID, а также:
* `ready`
* `endedSoon`
* `next`
* `selectRecommendation` id выбранной рекомендации
* TODO

## Методы TODO
* `on`, `once`, `off` аналогичны [EventEmitter node.js](https://nodejs.org/api/events.html)
* `setRenewCallback`
* `showRecommendations`
```javascript
player.once('endedSoon', () => player.showRecommendations([
    { id: 1, name: 'title', poster: '<url>' } // x3
]));
player.on('selectRecommendation', id => alert(`go to video with id ${id}`));
```

Статические
* `VenomPlayer.setVariables` обновить [cssVars](#css-vars) <a name="set-variables"></a>

## Список воспроизведения TODO

TODO
id
playlist: {seasons,id,ignoreLast},
season: 2,
episode: '13',

## Модуль рекламы
Настраивается с помощью опции `ads`. Поведение по умолчанию:

`start => pre roll ( => 10m => non linear => 5m => middle )`*

*поведение, заключенное в скобки, повторяется

```javascript
ads: {
    id: '', // берём в кабинете, для статистики
    volume: 0.3, // 30% громкости, чтобы сгладить контраст с контентом
    nonLinear: { // overlay
        url: 'https://...',
        offset: 10 * 60, // через 10 мин после старта и middle
        total: 2 // общее количество не больше 2 (по умолчанию не ограничено)
    },
    pre: {
        urls: ['https://...'] ссылка на vast
        maxImpressions: 2, // не больше 2 подряд
    },
    middle: {
        urls: ['https://...'],
        maxImpressions: 1, // не больше 1 подряд
        offset: 5 * 60, // через 5 мин после non-linear (overlay)
        total: 0 // общее количество не ограничено
    }
}
```

---

### text
| ключ | значение по умолчанию |
|---:|---|
|themeWrongVersion| версия темы не совпадает с версией плеера|
|themeLoadFailed| не удалось загрузить тему|

| ключ | значение по умолчанию |
|---:|---|	
|blocked| Видео заблокировано|
|mute| Отключить звук (m)|
|unMute| Включить звук (m)|
|pause| Пауза (пробел)|
|play| Смотреть (пробел)\nили клик в любом месте|
|fullscreenEnter| Полноэкранный режим (f)|
|fullscreenExit| Выход из полноэкранного режима (f)|
|pipIn| Режим "картинка в картинке"|
|pipOut| Выйти c режима "картинка в картинке"|
|settings| Настройки|
|quality| Качество|
|sound| Озвучка|
|speed| Скорость|
|cc| Субтитры|
|off| Откл|
|playlist| Список воспроизведения|
|emptyList| Список пуст.|
|Season| Сезон|
|season| сезон|
|episode| серия|
|episodes| Серии|
|next| Следующая|
|prev| Предыдущая|
|select| Выбрать|
|seconds| секунд|
|back| назад|
|nextIn| Следующая серия запустится через|
|report| Сообщить о проблеме|
|describeProblem| Опишите проблему|
|email| Email|
|cancel| Отмена|
|submit| Отправить|
|copy| Копировать|
|share| Поделиться ссылкой|
|shareWith| Поделиться с помощью|
|copyUrl| Копировать URL видео|
|copyWithTime| Копировать URL видео с привязкой ко времени|
|skipAd| Пропустить рекламу|
|after| \nможно через|
|sec| сек|
|online| Онлайн|
|goLive| В онлайн|
